package net.wanji.opt.controller.induce;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import freemarker.template.TemplateException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.opt.entity.GreenwaveInduces;
import net.wanji.opt.entity.GreenwaveInducesHist;
import net.wanji.opt.entity.InduceHist;
import net.wanji.opt.entity.InduceTemplate;
import net.wanji.opt.dto.induce.MessageParam;
import net.wanji.opt.service.induce.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.core.MediaType;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Api(value = "InduceSendController", description = "诱导屏-下发消息接口（对接易华录）")
@RestController
@RequestMapping("/induce")
@Slf4j
public class InduceSendController {

    @Autowired
    private InduceSendService induceSendService;

    @Autowired
    private InduceHistService induceHistService;

    @Autowired
    private InduceTemplateService induceTemplateService;

    @Autowired
    private GreenwaveInducesService greenwaveInducesService;

    @Autowired
    private GreenwaveInducesHistService greenwaveInducesHistService;

    @ApiOperation(value = "发送诱导信息", notes = "发送诱导信息", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @PostMapping("/send")
    public JsonViewObject send(@RequestBody MessageParam param) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        try {
            if(Objects.nonNull(param.getGreenId())) {
                LambdaQueryWrapper<GreenwaveInducesHist> greenwaveInducesHistQueryWrapper = new LambdaQueryWrapper<>();
                greenwaveInducesHistQueryWrapper.eq(GreenwaveInducesHist::getGreenId, param.getGreenId());
                List<GreenwaveInducesHist> greenwaveInducesHistList = greenwaveInducesHistService.list(greenwaveInducesHistQueryWrapper);
                if (Objects.nonNull(greenwaveInducesHistList) && greenwaveInducesHistList.size() > 0 && Objects.nonNull(greenwaveInducesHistList.get(0).getControlOptTimes()) && greenwaveInducesHistList.get(0).getControlOptTimes().split("|").length > 0 && DateUtil.isBetween(new Date(), DateUtil.parse(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), DateUtil.parse(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND))) {
                    param.setStartTime(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[0]);
                    param.setEndTime(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[1]);
                } else {
                    param.setStartTime(DateUtil.format(new Date(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                    param.setEndTime(DateUtil.format(System.currentTimeMillis() + 1000 * 60, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                }
            }
            induceSendService.send(param);
            return jsonView.success();
        } catch (Exception e) {
            log.info(e.getMessage());
            return jsonView.fail("发送诱导信息失败");
        }
    }

    @ApiOperation(value = "按绿波带编号批量发送诱导信息", notes = "按绿波带编号批量发送诱导信息", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @PostMapping("/batchSend")
    public JsonViewObject batchSend(@RequestBody MessageParam param) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        try {
            //查询绿波带信息，关联绿波路口，根据绿波路口查询关联诱导屏下发
            LambdaQueryWrapper<GreenwaveInduces> greenwaveInducesQueryWrapper = new LambdaQueryWrapper<>();
            greenwaveInducesQueryWrapper.eq(GreenwaveInduces::getGreenId, param.getGreenId());
            List<GreenwaveInduces> greenwaveInducesList=greenwaveInducesService.list(greenwaveInducesQueryWrapper);

            LambdaQueryWrapper<GreenwaveInducesHist> greenwaveInducesHistQueryWrapper = new LambdaQueryWrapper<>();
            greenwaveInducesHistQueryWrapper.eq(GreenwaveInducesHist::getGreenId, param.getGreenId());
            List<GreenwaveInducesHist> greenwaveInducesHistList=greenwaveInducesHistService.list(greenwaveInducesHistQueryWrapper);

            for(GreenwaveInduces greenwaveInduces:greenwaveInducesList)
            {
                LambdaQueryWrapper<InduceTemplate> induceTemplateQueryWrapper = new LambdaQueryWrapper<>();
                induceTemplateQueryWrapper.eq(InduceTemplate::getEquipCode, greenwaveInduces.getEquipCode());
                List<InduceTemplate> induceTemplateList=induceTemplateService.list(induceTemplateQueryWrapper);
                for(InduceTemplate induceTemplate:induceTemplateList) {
                    param.setTemplateId(induceTemplate.getId());
                    param.setInduceId(greenwaveInduces.getId());
                    param.setEquipCode(greenwaveInduces.getEquipCode());
                    param.setPlayorder(induceTemplateList.size());
                    param.setDuration(greenwaveInduces.getDuration());
                    if (Objects.nonNull(greenwaveInducesHistList) && greenwaveInducesHistList.size() > 0 && Objects.nonNull(greenwaveInducesHistList.get(0).getControlOptTimes()) && greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|").length > 0 && DateUtil.isBetween(new Date(), DateUtil.parse(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), DateUtil.parse(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND))) {
                        param.setStartTime(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[0]);
                        param.setEndTime(greenwaveInducesHistList.get(0).getControlOptTimes().split("\\|")[1]);
                    } else {
                        param.setStartTime(DateUtil.format(new Date(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                        param.setEndTime(DateUtil.format(System.currentTimeMillis() + 1000 * 60, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                    }
                    induceSendService.send(param);
                    Thread.sleep(1000);
                }
            }
            return jsonView.success();
        } catch (Exception e) {
            log.info(e.getMessage());
            return jsonView.fail("批量发布诱导信息失败");
        }
    }

    /**
     * 获取诱导发布图片
     */
    @ApiOperation(value = "根据设备编号获取诱导发布图片", notes = "根据设备编号获取诱导发布图片")
    @GetMapping(value = "/fileCode")
    public ResponseEntity<byte[]> ftpFileByCode(@RequestParam("equipCode") String equipCode) {
        // 根据 equipCode 获取文件名
        LambdaQueryWrapper<InduceHist> InduceHistQueryWrapper = new LambdaQueryWrapper<>();
        InduceHistQueryWrapper.eq(InduceHist::getEquipCode, equipCode);
        InduceHistQueryWrapper.orderByDesc(InduceHist::getCreateTime);
        InduceHist pictureFile = this.induceHistService.getOne(InduceHistQueryWrapper,false);
        if (Objects.isNull(pictureFile) && pictureFile.getFilePath() == null) {
            return ResponseEntity.status(HttpStatus.OK)
                    .body("无效的文件ID".getBytes(StandardCharsets.UTF_8));
        }
        try {
            // 获取ftp服务器图片
            byte[] imageBytes = this.induceSendService.downloadImage(pictureFile.getFilePath());
            if (imageBytes != null && imageBytes.length > 0) {
                // 设置响应头
                HttpHeaders headers = new HttpHeaders();
                headers.set("Content-Type", "image/jpeg"); // 图片类型
                headers.set("Content-Length", String.valueOf(imageBytes.length)); // 设置内容长度
                headers.set("Content-Disposition", "inline; filename=\"" + pictureFile.getSourceId() + ".bmp\""); // 设置文件名
                // 返回响应
                return new ResponseEntity<>(imageBytes, headers, HttpStatus.OK);
            } else {
                return ResponseEntity.status(HttpStatus.OK)
                        .body("未查询到对应文件".getBytes(StandardCharsets.UTF_8));
            }
        } catch (IOException e) {
            log.error("ftp文件下载失败: ", e.getMessage());
            return ResponseEntity.status(HttpStatus.OK)
                    .body("程序查询错误请联系管理员".getBytes(StandardCharsets.UTF_8));
        }
    }
    /**
     * 获取ftp图片
     */
    @GetMapping(value = "/file")
    public ResponseEntity<byte[]> ftpFile(@RequestParam("fileId") Long fileId) {
        // 根据 fileId 获取文件名
        InduceHist pictureFile = this.induceHistService.getById(fileId);
        if (pictureFile == null) {
            return ResponseEntity.status(HttpStatus.OK)
                    .body("无效的文件ID".getBytes(StandardCharsets.UTF_8));
        }
        try {
            // 获取ftp服务器图片
            byte[] imageBytes = this.induceSendService.downloadImage(pictureFile.getFilePath());
            if (imageBytes != null && imageBytes.length > 0) {
                // 设置响应头
                HttpHeaders headers = new HttpHeaders();
                headers.set("Content-Type", "image/jpeg"); // 图片类型
                headers.set("Content-Length", String.valueOf(imageBytes.length)); // 设置内容长度
                headers.set("Content-Disposition", "inline; filename=\"" + pictureFile.getSourceId() + ".bmp\""); // 设置文件名
                // 返回响应
                return new ResponseEntity<>(imageBytes, headers, HttpStatus.OK);
            } else {
                return ResponseEntity.status(HttpStatus.OK)
                        .body("未查询到对应文件".getBytes(StandardCharsets.UTF_8));
            }
        } catch (IOException e) {
            log.error("ftp文件下载失败: ", e.getMessage());
            return ResponseEntity.status(HttpStatus.OK)
                    .body("程序查询错误请联系管理员".getBytes(StandardCharsets.UTF_8));
        }
    }

}
