package net.wanji.opt.controller.signalopt;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataRealtimePO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.service.CrossIndexService;
import net.wanji.opt.vo.AIOptResultVO;
import net.wanji.opt.vo.CrossOptResult;
import net.wanji.opt.vo.crossStatusCountVO;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2024/11/25 19:10
 */
@Api(value = "CrossIndexController", description = "路口指标控制器")
@RequestMapping("/crossIndex")
@RestController
public class CrossIndexController {

    @Resource
    private CrossIndexService crossIndexService;

    @ApiOperation(value = "路口信息查询", notes = "路口信息查询", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossInfoList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = TBaseCrossInfo.class),
    })
    public JsonViewObject crossInfoList() {
        List<TBaseCrossInfo> baseCrossInfoPOS = crossIndexService.crossInfoList();
        return JsonViewObject.newInstance().success(baseCrossInfoPOS);
    }

    @ApiOperation(value = "路口方向指标", notes = "路口方向指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossDirIndex",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossDirIndex(@RequestBody CrossIdBO crossIdBO) {
        Map<Integer, CrossDirDataRealtimePO> result = crossIndexService.crossDirIndex(crossIdBO);
        return JsonViewObject.newInstance().success(result);
    }

    @ApiOperation(value = "路口指标", notes = "路口指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossIndex",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossIndex(@RequestBody CrossIdBO crossIdBO) {
        CrossDataRealtimePO crossDataRealtimePO = crossIndexService.crossIndex(crossIdBO);
        return JsonViewObject.newInstance().success(crossDataRealtimePO);
    }

    @ApiOperation(value = "方案优化曲线", notes = "优化监测-方案优化曲线", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossOptResultList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossOptResultList(@RequestBody CrossIdBO crossIdBO) {
        List<CrossOptResult> results = null;
        try {
            results = crossIndexService.crossOptResultList(crossIdBO);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("方案优化曲线查询异常");
        }
        return JsonViewObject.newInstance().success(results);
    }


    @ApiOperation(value = "AI路口", notes = "优化监测-AI路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossAIList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossAIList() {
        List<AIOptResultVO> results = null;
        try {
            results = crossIndexService.crossAIList();
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("优化监测-AI路口");
        }
        return JsonViewObject.newInstance().success(results);
    }

    @ApiOperation(value = "问题诊断", notes = "优化监测-问题诊断", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossStatusCount")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossStatusCount(String crossId) {
        crossStatusCountVO result = new crossStatusCountVO();
        try {
            result = crossIndexService.crossStatusCount(crossId);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("优化监测-问题诊断");
        }
        return JsonViewObject.newInstance().success(result);
    }

}
