package net.wanji.opt.service;

import com.alibaba.fastjson.JSONObject;
import net.wanji.common.framework.dubbointerface.BaseDubboInterface;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.vo.GreenWaveRunStateVO;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 绿波历史数据
 * </p>
 *
 * @Author fengyi
 * @Date 2024-11-18
 */
public interface GreenwaveHistProvider extends BaseDubboInterface<GreenwaveHist> {
    /**
     * @param greenId
     * @param startTime
     * @param endTime
     * @param type
     * @return
     * @throws DubboProviderException
     */
    JSONObject findStatisticIndex(String greenId, String startTime, String endTime, String type) throws DubboProviderException;

    /**
     * 干线路口信息
     *
     * @param greenId
     * @return
     * @throws DubboProviderException
     */
    List<Map<String, Object>> findGreenCross(Integer greenId) throws DubboProviderException;

    /**
     * 非协调方向路口指标统计
     *
     * @param crossId   路口ID
     * @param greenDir  协调方向
     * @param startTime 开始时间
     * @param endTime   截止时间
     * @return
     */
    JSONObject findUnCoordinateCrossIndex(String crossId, String greenDir, String startTime, String endTime) throws DubboProviderException;

    /**
     * 路口方向级不同时间粒度指标分析
     * @param crossId
     * @param startTime
     * @param endTime
     * @param groupType
     * @return
     * @throws DubboProviderException
     */
    JSONObject findGreenWaveCrossDirIndex(String crossId, String startTime, String endTime, String groupType) throws DubboProviderException;

    /**
     * 路口级不同时间粒度指标分析
     * @param crossId
     * @param startTime
     * @param endTime
     * @param groupType
     * @return
     * @throws DubboProviderException
     */
    JSONObject findGreenWaveCrossIndex(String crossId, String startTime, String endTime, String groupType) throws DubboProviderException;

    /**
     * 干线运行状态查询
     * @param greenId
     * @param startTime
     * @param endTime
     * @return
     */
    List<GreenWaveRunStateVO> findGreenWaveRunState(Integer greenId,String crossId, String startTime, String endTime,String eventType) throws DubboProviderException;
}
