package net.wanji.opt.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.CrossDirDataRealtimeMapper;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataRealtimePO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.opt.common.enums.CrossOptStrategyEnum;
import net.wanji.opt.common.enums.OptStatusEnum;
import net.wanji.opt.common.enums.StrategyControlEnum;
import net.wanji.opt.dao.mapper.StrategyCrossResultMapper;
import net.wanji.opt.service.CrossIndexService;
import net.wanji.opt.synthesis.pojo.StrategyCrossResultEntity;
import net.wanji.opt.vo.AIOptResultVO;
import net.wanji.opt.vo.CrossOptResult;
import net.wanji.opt.vo.crossStatusCountVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/25 19:12
 */
@Service
@Slf4j
public class CrossIndexServiceImpl implements CrossIndexService {

    @Resource
    private CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;

    @Override
    public Map<Integer, CrossDirDataRealtimePO> crossDirIndex(CrossIdBO crossIdBO) {
        List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = crossDirDataRealtimeMapper.selectBycrossId(crossIdBO.getCrossId());
        if (!CollectionUtils.isEmpty(crossDirDataRealtimePOS)) {
            Map<Integer, CrossDirDataRealtimePO> dirDataMap = new HashMap<>(crossDirDataRealtimePOS.size());
            for (CrossDirDataRealtimePO crossDirDataRealtimePO : crossDirDataRealtimePOS) {
                dirDataMap.put(crossDirDataRealtimePO.getDirType(), crossDirDataRealtimePO);
            }
            return dirDataMap;
        }
        return null;
    }

    @Override
    public CrossDataRealtimePO crossIndex(CrossIdBO crossIdBO) {
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossIdBO.getCrossId());
        return crossDataRealtimePO;
    }


    @Override
    public List<TBaseCrossInfo> crossInfoList() {
        LambdaQueryWrapper<TBaseCrossInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TBaseCrossInfo::getIsSignal, 1);
        List<TBaseCrossInfo> baseCrossInfoPOS = baseCrossInfoMapper.selectList(queryWrapper);
        return baseCrossInfoPOS;
    }

    @Override
    public List<CrossOptResult> crossOptResultList(CrossIdBO crossIdBO) throws Exception{
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossIdBO.getCrossId());
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossIdBO.getCrossId());
        queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, startOfDay);
        List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectList(queryWrapper);
        List<CrossOptResult> crossOptResults = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            for (StrategyCrossResultEntity entity : list) {
                CrossOptResult crossOptResult = new CrossOptResult();
                Date date =DateUtil.parse(entity.getIssueTime(), "yyyy-MM-dd HH:mm:ss");
                crossOptResult.setTimeStamp(date);
                String timingPlan = entity.getTimingPlan();
                Integer countDown = entity.getCountDown();
                if (entity.getCurrentAlgo() == 2) {
                    countDown = 5;
                }
                crossOptResult.setCountDown(countDown);
                Integer currentAlgo = entity.getCurrentAlgo();
                crossOptResult.setStrategy(currentAlgo);
                crossOptResult.setStrategyName(CrossOptStrategyEnum.getDesc(currentAlgo));
                crossOptResults.add(crossOptResult);
            }
        }
        return crossOptResults;
    }

    @Override
    public List<AIOptResultVO> crossAIList() {
        List<AIOptResultVO> results = new ArrayList<>();
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, startOfDay);
        List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(list)) {
            // 查询出表中当天最新数据
            Map<String, Optional<StrategyCrossResultEntity>> crossTimeMap = list.stream().collect(Collectors.groupingBy(StrategyCrossResultEntity::getCrossId,
                    Collectors.maxBy(Comparator.comparing(StrategyCrossResultEntity::getIssueTime))));
            List<StrategyCrossResultEntity> crossMaxTimeList = crossTimeMap.values().stream().map(Optional::get).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(crossMaxTimeList)) {
                for (StrategyCrossResultEntity resultEntity : crossMaxTimeList) {
                    AIOptResultVO aiOptResultVO = new AIOptResultVO();
                    aiOptResultVO.setId(resultEntity.getCrossId());
                    aiOptResultVO.setName(resultEntity.getCrossName());
                    Integer currentAlgo = resultEntity.getCurrentAlgo();
                    aiOptResultVO.setStrategy(currentAlgo);
                    // 失衡 均衡调控，其他效率提升
                    aiOptResultVO.setOptMethod(Objects.equals(1, currentAlgo) ? StrategyControlEnum.ONE.getMethod() : StrategyControlEnum.TWO.getMethod());
                    // 失衡 优化中，其他，优化完
                    Integer optStatus = Objects.equals(1, currentAlgo) ? OptStatusEnum.ONE.getCode() : OptStatusEnum.TWO.getCode();
                    aiOptResultVO.setOptStatus(optStatus);
                    aiOptResultVO.setOptStatusName(Objects.equals(1, currentAlgo) ? OptStatusEnum.ONE.getDesc() : OptStatusEnum.TWO.getDesc());
                    results.add(aiOptResultVO);
                }
            }
        }
        Collections.sort(results, Comparator.comparingInt(AIOptResultVO::getOptStatus));
        return results;
    }

    @Override
    public crossStatusCountVO crossStatusCount(String crossId) {
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossId);
        queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, startOfDay);
        List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectList(queryWrapper);
        crossStatusCountVO crossStatusCountVO = new crossStatusCountVO();
        if (!CollectionUtils.isEmpty(list)) {
            Map<Integer, List<StrategyCrossResultEntity>> listMap = list.stream().collect(Collectors.groupingBy(StrategyCrossResultEntity::getCurrentAlgo));
            for (Map.Entry<Integer, List<StrategyCrossResultEntity>> entry : listMap.entrySet()) {
                Integer currentAlgo = entry.getKey();
                List<StrategyCrossResultEntity> value = entry.getValue();
                if (Objects.equals(currentAlgo, CrossOptStrategyEnum.ONE.getCode())) {
                    crossStatusCountVO.setPhaseEmptyCount(value.size());
                }
                if (Objects.equals(currentAlgo, CrossOptStrategyEnum.TWO.getCode())) {
                    crossStatusCountVO.setUnbalanceCount(value.size());
                }
                if (Objects.equals(currentAlgo, CrossOptStrategyEnum.THREE.getCode())) {
                    crossStatusCountVO.setOverFlowCount(value.size());
                }
            }
        }
        return crossStatusCountVO;
    }
}
