package net.wanji.opt.service.impl;


import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.dubbointerface.impl.BaseDubboInterfaceImpl;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.mapper.BaseInterfaceMapper;
import net.wanji.common.gts.Tools;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.service.GreenwaveHistProvider;
import net.wanji.opt.vo.GreenWaveRunStateVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboService;
import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Random;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;


/**
 * <p>
 * 绿波历史数据
 * </p>
 *
 * @Author fengyi
 * @Date 2024-11-18
 */
@Slf4j
@Component
@DubboService
public class GreenwaveHistProviderImpl extends BaseDubboInterfaceImpl<GreenwaveHist> implements GreenwaveHistProvider {

    @Resource
    private GreenwaveHistoryMapper greenwaveHistoryMapper;

    @Override
    public BaseInterfaceMapper<GreenwaveHist> getBaseInterfaceMapper() {
        return this.greenwaveHistoryMapper;
    }


    @Override
    public JSONObject findStatisticIndex(String greenId, String startTime, String endTime, String type) {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);

        List<GreenwaveHist> list = greenwaveHistoryMapper.findStatisticIndex(params);
        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有进口数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<String, List<GreenwaveHist>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getRoadDirection(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<String, List<GreenwaveHist>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();

            //按方向排序
            List<GreenwaveHist> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (GreenwaveHist po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirection());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("type", type);
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("directionList", new int[]{0, 1, 2});

        return jsonObject;

    }

    @Override
    public List<Map<String, Object>> findGreenCross(Integer greenId) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);

        List<Map<String, Object>> list = greenwaveHistoryMapper.findGreenCross(params);
        String dirs = list.stream().map(o -> o.get("inDir").toString()).distinct().collect(Collectors.joining(","));


        return list;
    }

    @Override
    public JSONObject findUnCoordinateCrossIndex(String crossId, String greenDir, String startTime, String endTime) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("greenDir", greenDir);

        List<GreenwaveHist> list = greenwaveHistoryMapper.findUnCoordinateCrossIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有进口数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<Integer, List<GreenwaveHist>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getDirection(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<Integer, List<GreenwaveHist>> entry : groupByDir.entrySet()) {
            Integer key = entry.getKey();

            //按方向排序
            List<GreenwaveHist> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (GreenwaveHist po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirection());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);

        return jsonObject;

    }

    @Override
    public JSONObject findGreenWaveCrossDirIndex(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossDirDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossDirIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有进口数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<Integer, List<CrossDirDataHistPO>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getDirType(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : groupByDir.entrySet()) {
            Integer key = entry.getKey();

            //按方向排序
            List<CrossDirDataHistPO> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (CrossDirDataHistPO po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirType());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("dirList", groupByDir.keySet());

        return jsonObject;
    }


    @Override
    public JSONObject findGreenWaveCrossIndex(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        for (CrossDataHistPO po : list) {
            //提取时间
            sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", list);

        return jsonObject;
    }

    @Override
    public List<GreenWaveRunStateVO> findGreenWaveRunState(Integer greenId,String crossId, String startTime, String endTime,String eventType) {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("crossId", crossId);

        if (Objects.nonNull(eventType)) {
            String[] arr = StringUtils.split(eventType, ",");
            params.put("eventTypeList", Arrays.asList(arr));
        }

        int[] stateList = new int[]{705,706};

        List<GreenWaveRunStateVO> list = greenwaveHistoryMapper.findGreenWaveRunState(params);
        Map<Integer,GreenWaveRunStateVO> groupList = list.stream().collect(Collectors.toMap(o -> o.getState(), o -> o));



        DateTime startDateTime = DateTime.parse(startTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));
        DateTime endDateTime = DateTime.parse(endTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

        for (int state : stateList) {
            if (!groupList.containsKey(state)) {
                GreenWaveRunStateVO tmp = new GreenWaveRunStateVO();
                tmp.setState(state);
                tmp.setGreenId(greenId);
                list.add(tmp);
            }
        }
        int totalTime = Seconds.secondsBetween(startDateTime,endDateTime).getSeconds();
        for (GreenWaveRunStateVO vo : list ) {
//            if (vo.getDuration()<=0) {
//                vo.setDuration(getRandomValue(totalTime / 10, 120));
//            }
            vo.setTotalTime(totalTime);
        }

        return  list;
    }

    /**
     * 获取随机数
     * @param max
     * @param min
     * @return
     */
    public static  int getRandomValue(int max,int min){
        Random random = new Random();
        int num =random.nextInt(max+1-min)+min;
        return num;
    }

}
