package net.wanji.opt.service.signalopt.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenwaveCrossPO;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.opt.cache.BaseCrossInfoCache;
import net.wanji.opt.cache.GreenWaveInfoCache;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.common.enums.GreenWaveInDirEnum;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.service.signalopt.GreenBeltInfoService;
import net.wanji.opt.vo.*;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/12/02 13:42
 */
@Service
@Slf4j
public class GreenBeltServiceImpl implements GreenBeltInfoService {

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    @Resource
    private GreenwaveHistMapper greenwaveHistMapper;
    @Resource
    private GreenwaveCrossMapper greenwaveCrossMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private BaseCrossInfoCache baseCrossInfoCache;
    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;


    @Override
    public GreenBeltCrossDetailVO greenBeltCrossDetailList(Integer greenId) throws Exception {
        List<GreenwaveCrossPO> crossPOS = greenwaveCrossMapper.selectByGreenwaveId(greenId);
        List<CrossDataRealtimePO> realtimePOS = crossDataRealtimeMapper.selectAll();
        GreenBeltCrossDetailVO greenBeltCrossDetailVO = new GreenBeltCrossDetailVO();
        LambdaQueryWrapper<GreenwaveRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(GreenwaveRealtimePO::getGreenId, greenId);
        if (!CollectionUtils.isEmpty(crossPOS) && !CollectionUtils.isEmpty(realtimePOS)) {
            List<GreenBeltCrossDetailVO.CrossDetail> detailList = new ArrayList<>();
            for (GreenwaveCrossPO crossPO : crossPOS) {
                String crossId = crossPO.getCrossId();
                GreenBeltCrossDetailVO.CrossDetail crossDetail = new GreenBeltCrossDetailVO.CrossDetail();
                for (CrossDataRealtimePO realtimePO : realtimePOS) {
                    if (StringUtils.equalsIgnoreCase(crossPO.getCrossId(), realtimePO.getCrossId())) {
                        String crossName = baseCrossInfoCache.getCrossName(crossId);
                        crossDetail.setCrossName(crossName);
                        crossDetail.setCrossIndex(realtimePO.getTrafficIndex());
                        detailList.add(crossDetail);
                    }
                }
            }
            List<GreenwaveRealtimePO> greenRealTimes = greenwaveRealtimeMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(greenRealTimes)) {
                double asDouble = greenRealTimes.stream().mapToDouble(GreenwaveRealtimePO::getTrafficIndex).average().getAsDouble();
                greenBeltCrossDetailVO.setGreenIndex(asDouble);
            }
            greenBeltCrossDetailVO.setDetailList(detailList);
            Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
            GreenwaveInfoPO greenwaveInfoPO = greenWaveMap.get(greenId);
            greenBeltCrossDetailVO.setGreenName(greenwaveInfoPO.getName());
            greenBeltCrossDetailVO.setCrossNum(detailList.size());

        }
        return greenBeltCrossDetailVO;
    }

    @Override
    public List<GreenBeltKeyCrossFlowTimeVO> greenBeltKeyCrossFlowTime(Integer greenId) throws Exception {
        try {
            List<GreenBeltKeyCrossFlowTimeVO> results = new ArrayList<>();
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            // 最少有一条数据
            List<GreenBeltKeyCrossFlowTimeVO.Detail> optResults = getGreenOptList(greenId, startOfDay);
            GreenBeltKeyCrossFlowTimeVO.Detail optDetail = optResults.get(0);
            String crossId = optDetail.getCrossId();
            GreenwaveInfoPO greenwaveInfoPO = GreenWaveInfoCache.greenWaveMap.get(greenId);
            String greenDir = greenwaveInfoPO.getGreenDir();
            String[] split = greenDir.split(",");
            List<Integer> dirList = new ArrayList<>();
            for (String s : split) {
                dirList.add(Integer.valueOf(s));
            }
            Instant instant = startOfDay.atZone(ZoneId.systemDefault()).toInstant();
            int start = (int) (instant.toEpochMilli() / 1000) + 300;
            int end = (int) (System.currentTimeMillis() / 1000) + 300;
            List<CrossDirDataHistPO> dirDataHistPOS = crossDirDataHistMapper.selectDirDataList(crossId, dirList, start, end);
            if (!CollectionUtils.isEmpty(dirDataHistPOS)) {
                Double greenTimeRatio = 0.0;
                List<GreenBeltKeyCrossFlowTimeVO.Detail> details = new ArrayList<>();
                for (CrossDirDataHistPO dirDataHistPO : dirDataHistPOS) {
                    GreenBeltKeyCrossFlowTimeVO.Detail detail = new GreenBeltKeyCrossFlowTimeVO.Detail();
                    detail.setCrossId(crossId);
                    String crossName = baseCrossInfoCache.getCrossName(crossId);
                    detail.setCrossName(crossName);
                    Integer dirType = dirDataHistPO.getDirType();
                    detail.setDir(String.valueOf(dirType));
                    detail.setFlow(dirDataHistPO.getFlow());
                    detail.setStartTime(dirDataHistPO.getStartTime());
                    detail.setGreenTimeRatio(greenTimeRatio);
                    for (GreenBeltKeyCrossFlowTimeVO.Detail optResult : optResults) {
                        Integer curDir = GreenBeltDirEnum.getInDir(optResult.getDir());
                        if (StringUtils.equalsIgnoreCase("cancel", optResult.getDir()) && Objects.equals(curDir, dirType)
                                && dirDataHistPO.getStartTime().getTime() == optResult.getStartTime().getTime()) {
                            greenTimeRatio = 0.0;
                        }
                        if (StringUtils.equalsIgnoreCase(crossId, optResult.getCrossId()) && Objects.equals(curDir, dirType)
                                && dirDataHistPO.getStartTime().getTime() == optResult.getStartTime().getTime()) {
                            detail.setGreenTimeRatio(optResult.getGreenTimeRatio());
                            greenTimeRatio = optResult.getGreenTimeRatio();
                        }

                    }
                    details.add(detail);
                }
                if (!CollectionUtils.isEmpty(details)) {
                    Map<String, List<GreenBeltKeyCrossFlowTimeVO.Detail>> dirMap = details.stream().collect(Collectors.groupingBy(GreenBeltKeyCrossFlowTimeVO.Detail::getDir));
                    for (Map.Entry<String, List<GreenBeltKeyCrossFlowTimeVO.Detail>> entry : dirMap.entrySet()) {
                        String dir = entry.getKey();
                        GreenBeltKeyCrossFlowTimeVO greenBeltKeyCrossFlowTimeVO = new GreenBeltKeyCrossFlowTimeVO();
                        List<GreenBeltKeyCrossFlowTimeVO.Detail> value = entry.getValue();
                        String crossName = "";
                        if (!CollectionUtils.isEmpty(value)) {
                            GreenBeltKeyCrossFlowTimeVO.Detail detail = value.get(0);
                            crossName = detail.getCrossName();
                        }
                        greenBeltKeyCrossFlowTimeVO.setCrossName(crossName);
                        greenBeltKeyCrossFlowTimeVO.setDirName(GreenBeltDirEnum.getInDirName(Integer.valueOf(dir)));
                        List<GreenBeltKeyCrossFlowTimeVO.Detail> collect = value.stream().sorted(Comparator.comparing(GreenBeltKeyCrossFlowTimeVO.Detail::getStartTime)).collect(Collectors.toList());
                        greenBeltKeyCrossFlowTimeVO.setDetailList(collect);
                        results.add(greenBeltKeyCrossFlowTimeVO);
                    }
                }
            }

            // 关键路口历史数据
            return results;
        } catch (Exception e) {
            log.error("绿波关键路口流量绿信比查询异常：", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取绿波优化列表
     *
     * @param greenId
     * @param startOfDay
     * @return
     * @throws JsonProcessingException
     */
    private List<GreenBeltKeyCrossFlowTimeVO.Detail> getGreenOptList(Integer greenId, LocalDateTime startOfDay) throws Exception {
        List<GreenBeltKeyCrossFlowTimeVO.Detail> optResults = null;
        try {
            optResults = new ArrayList<>();
            // 绿波优化数据
            List<StrategyGreenOptHistEntity> entities = new ArrayList<>();
            LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
            entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            if (CollectionUtils.isEmpty(entities)) {
                // 如果路口没有绿波，获取当前绿波最大的一条数据
                LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapperMaxId = new LambdaQueryWrapper<>();
                queryWrapperMaxId.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
                queryWrapperMaxId.ge(StrategyGreenOptHistEntity::getControlMethod, 1);
                queryWrapperMaxId.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay.minusDays(1));
                queryWrapperMaxId.orderByDesc(StrategyGreenOptHistEntity::getControlTime);
                queryWrapperMaxId.last("limit 1");
                entities = strategyGreenOptHistMapper.selectList(queryWrapperMaxId);
            }
            // 一个绿波只有一个关键路口
            if (!CollectionUtils.isEmpty(entities)) {
                for (StrategyGreenOptHistEntity entity : entities) {
                    GreenBeltKeyCrossFlowTimeVO.Detail detail = new GreenBeltKeyCrossFlowTimeVO.Detail();
                    String keyCross = entity.getKeyCross();
                    detail.setCrossId(keyCross);
                    detail.setDir(entity.getDir());
                    Date controlTime = get5MinuteDate(entity.getControlTime());
                    detail.setStartTime(controlTime);
                    String crossGreenDetail = entity.getCrossGreenDetail();
                    Integer cycle = entity.getCycle();
                    ObjectMapper mapper = JacksonUtils.getInstance();
                    List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetails = mapper.readValue(crossGreenDetail, new TypeReference<List<GreenBeltInfoVO.CrossGreenDetail>>() {
                    });
                    if (!CollectionUtils.isEmpty(crossGreenDetails)) {
                        List<GreenBeltInfoVO.CrossGreenDetail> keyCrossDetail = crossGreenDetails.stream().filter(details -> StringUtils.equalsIgnoreCase(keyCross, details.getCrossId())).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(keyCrossDetail)) {
                            for (GreenBeltInfoVO.CrossGreenDetail curKeyCross : keyCrossDetail) {
                                String currentCrossId = curKeyCross.getCrossId();
                                if (StringUtils.equalsIgnoreCase(keyCross, currentCrossId)) {
                                    Double phaseStartTime = curKeyCross.getPhaseStartTime();
                                    Double phaseEndTime = curKeyCross.getPhaseEndTime();
                                    Double greenTimeRatio = (phaseEndTime - phaseStartTime) / cycle;
                                    detail.setGreenTimeRatio(greenTimeRatio);
                                }
                            }
                        }
                    }
                    // 设置中间变量 优化结束
                    if (entity.getControlMethod() <= 0) {
                        detail.setDir("cancel");
                    }
                    optResults.add(detail);
                }
            }
        } catch (Exception e) {
            log.error("绿波优化列表数据异常；", e);
            throw new Exception(e);
        }
        return optResults;
    }


    @Override
    public List<GreenBeltSpeedWidthVO> greenBeltSpeedWidth(Integer greenId) throws Exception {
        try {
            List<GreenBeltSpeedWidthVO> results = new ArrayList<>();
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            // 绿波历史数据
            LambdaQueryWrapper<GreenwaveHistPO> greenWrapper = new LambdaQueryWrapper<>();
            greenWrapper.eq(GreenwaveHistPO::getGreenId, greenId);
            greenWrapper.ge(GreenwaveHistPO::getStartTime, startOfDay);
            List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectList(greenWrapper);
            // 绿波优化数据
            LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlMethod, 1);
            List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(greenwaveHistPOS)) {
                Map<String, List<GreenwaveHistPO>> greenDirMap = greenwaveHistPOS.stream().collect(Collectors.groupingBy(GreenwaveHistPO::getRoadDirection));
                for (Map.Entry<String, List<GreenwaveHistPO>> entry : greenDirMap.entrySet()) {
                    String dir = entry.getKey();
                    GreenBeltSpeedWidthVO greenBeltSpeedWidthVO = new GreenBeltSpeedWidthVO();
                    greenBeltSpeedWidthVO.setDirName(GreenBeltDirEnum.getDesc(dir));
                    List<GreenBeltSpeedWidthVO.Detail> details = new ArrayList<>();
                    List<GreenwaveHistPO> value = entry.getValue();
                    double greenWidthTime = 0.0;
                    for (GreenwaveHistPO histPO : value) {
                        Integer curGreenId = histPO.getGreenId();
                        Date curStartTime = histPO.getStartTime();
                        String curStartTimeStr = DateUtil.format(curStartTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                        Date startTimeminuteDate = get5MinuteDate(curStartTimeStr);
                        Double speed = histPO.getSpeed();
                        GreenBeltSpeedWidthVO.Detail detail = new GreenBeltSpeedWidthVO.Detail();
                        for (StrategyGreenOptHistEntity entity : entities) {
                            Integer optGreenId = entity.getGreenId();
                            String controlTime = entity.getControlTime();
                            String optDir = entity.getDir();
                            Date minuteDate = get5MinuteDate(controlTime);
                            if (Objects.equals(curGreenId, optGreenId) && StringUtils.equalsIgnoreCase(optDir, dir)
                                    && minuteDate.getTime() == startTimeminuteDate.getTime()) {
                                greenWidthTime = entity.getGreenWidthTime();
                            }
                            if (entity.getControlMethod() <= 0) {
                                greenWidthTime = 0.0;
                            }
                        }
                        detail.setStartTime(curStartTime);
                        detail.setSpeed(speed);
                        detail.setWidth(greenWidthTime);
                        detail.setDir(dir);
                        details.add(detail);
                    }
                    greenBeltSpeedWidthVO.setDetailList(details);
                    results.add(greenBeltSpeedWidthVO);
                }
            }
            return results;
        } catch (Exception e) {
            log.error("绿波带宽曲线异常：", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 将下发时间按五分钟取整
     *
     * @param controlTime
     * @return
     */
    public static Date get5MinuteDate(String controlTime) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.parse(controlTime, formatter);
        // 获取当前分钟
        int minute = dateTime.getMinute();
        // 计算需要补充的分钟数（如果 minute % 5 != 0）
        int minutesToAdd = (5 - minute % 5) % 5;
        // 添加这些分钟并返回新的时间
        LocalDateTime localDateTime = dateTime.plusMinutes(minutesToAdd).withSecond(0).withNano(0);
        ZoneId zoneId = ZoneId.systemDefault();
        Date minuteDate = Date.from(localDateTime.atZone(zoneId).toInstant());
        return minuteDate;
    }

    @Override
    public List<GreenBeltFlowStopTimeVO> greenBeltCrossDetailHist(Integer greenId) throws Exception {
        try {
            List<String> crossIds = greenwaveInfoMapper.selectCrossIdsById(greenId);
            GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenId);
            long currentTimeMillis = System.currentTimeMillis();
            int startSecond = ((int) (currentTimeMillis / 1000)) - 3600;
            int endSecond = (int) (currentTimeMillis / 1000);
            List<GreenBeltFlowStopTimeVO> results = new ArrayList<>();
            if (Objects.nonNull(greenwaveInfoPO)) {
                String greenDir = greenwaveInfoPO.getGreenDir();
                String[] split = greenDir.split(",");
                List<Integer> dirList = new ArrayList<>(split.length);
                for (String s : split) {
                    dirList.add(Integer.valueOf(s));
                }
                List<CrossDirDataHistPO> dirDataHistPOS = crossDirDataHistMapper.selectByCrossIdsDirsAndTimestamp(crossIds, dirList, startSecond, endSecond);
                if (!CollectionUtils.isEmpty(dirDataHistPOS)) {
                    Map<Date, List<CrossDirDataHistPO>> dateMap = dirDataHistPOS.stream().collect(Collectors.groupingBy(CrossDirDataHistPO::getStartTime));
                    for (Map.Entry<Date, List<CrossDirDataHistPO>> entry : dateMap.entrySet()) {
                        Date date = entry.getKey();
                        List<CrossDirDataHistPO> value = entry.getValue();
                        GreenBeltFlowStopTimeVO greenBeltFlowStopTimeVO = new GreenBeltFlowStopTimeVO();
                        greenBeltFlowStopTimeVO.setStartTime(date);
                        List<GreenBeltFlowStopTimeVO.FlowStopTimeDetail> detailList = new ArrayList<>();
                        if (!CollectionUtils.isEmpty(value)) {
                            for (CrossDirDataHistPO crossDirDataHistPO : value) {
                                GreenBeltFlowStopTimeVO.FlowStopTimeDetail detail = new GreenBeltFlowStopTimeVO.FlowStopTimeDetail();
                                Integer dirType = crossDirDataHistPO.getDirType();
                                detail.setGreenDir(GreenWaveInDirEnum.getDesc(dirType));
                                detail.setCrossId(crossDirDataHistPO.getCrossId());
                                detail.setFlow(crossDirDataHistPO.getFlow());
                                detail.setStopTimes(crossDirDataHistPO.getStopTimes());
                                detail.setSpeed(crossDirDataHistPO.getSpeed());
                                detailList.add(detail);
                            }
                        }
                        greenBeltFlowStopTimeVO.setDetailList(detailList);
                        results.add(greenBeltFlowStopTimeVO);
                    }
                }
                Collections.sort(results, Comparator.comparing(GreenBeltFlowStopTimeVO::getStartTime));
                return results;
            }
            return Collections.EMPTY_LIST;
        } catch (Exception e) {
            log.error("绿波协调方向路口流量停车次数查询异常：", e);
            throw new RuntimeException(e);
        }
    }
}
